/**
 * Use it like so:

display( function() { return getFriendLeaderboardInstance('tutorial-0', 'Tutorial').update(); }); 


 * Or like so, if you have all the highscores already (sorted):

display( function() { return getFriendLeaderboardInstance('tutorial-0', 'Tutorial').updateWithHighscores(
		new Array( 
			new Highscore('1321307561', 'fb1321307561', 'Tobias Andersson', '13551'),
			new Highscore('5134337', 'fb5134337', 'Allyn Bencen', '40976')
		)
	); 
});

 * Or like so, if you have all the friends who authorized the app, but not the highscores (no need to sort):
 * The first argument is an array of facebook ids
 * The second argument is a dictionary that can translate from id to name

display( function() { return getFriendLeaderboardInstance('tutorial-0', 'Tutorial').updateWithFriends(
		new Array( '1321307561', '5134337'),
		{ '1321307561' : 'Tobias Andersson', '5134337' : 'Allyn Bencen' }
	); 
});

 * 
 */

Date.defineFormat('racetime', function(date){
	return date.get('min') + date.format(':%S.') + date.format('%L').substring(0,2);
});

var friendLeaderboardInstance = null;

function getFriendLeaderboardInstance(board, name) {
	if (friendLeaderboardInstance !== null && friendLeaderboardInstance.board == board) {
		friendLeaderboardInstance.name = name;
		return friendLeaderboardInstance;
	} else {
		return new FriendLeaderboard(board, name);
	}
}

var FriendLeaderboard = new Class({
	initialize: function(board, name) {
		this.board = board; // The level
		this.name = name; // The header (level name)
		this.highscores = []; // An Array of HighScore
		this.element = new Element('div');
		friendLeaderboardInstance = this;
	},
	/**
	 * Updates the leaderboards, consider using one of the other update functions if you already have the user's friends or even
	 * all the highscores
	 */
	update: function(onUpdated) {
		console.log('Running FriendLeaderboard.update()');
		// get my friends who have authorized the FB app and get me a leaderboard of those
		facebook('me/friends/?fields=installed,name', function(fbresponse) {
			console.log('FriendLeaderboard: Got facebook response');

			var facebookIds = [];
			var idToNames = [];
			var i = 0;

			// which friends have authorized the app?
			Array.each(fbresponse.data, function(friend, index) {
				if (friend.installed || config.debug) {
					facebookIds[i] = friend.id;
					idToNames[''+friend.id] = friend.name;
					i++;
				}
			});

			// Add our own id to show up in the leaderboard
			facebookIds[i] = user.facebookId;
			idToNames[''+user.facebookId] = user.name;

			this.updateWithFriends(facebookIds, idToNames, onUpdated);

		}.bind(this));

		return this;
	},
	/**
	 * Call this if you already have all the friends who authorized the app
	 */
	updateWithFriends: function(facebookIds, idToNames, onUpdated) {
		console.log('Running FriendLeaderboard.updateWithFriends()');
		var req = new GameRequest('getfriendleaderboard', function(response) {
			console.log('FriendLeaderboard: Got gameRequest response');
			this.highscores = [];
			var iHaveScore = false;
			var facebookId;
			for (var i = 0 ; i < response.highscores.length ; i++) {
				facebookId = response.highscores[i].facebookId;
				this.highscores[i] = new Highscore(
						facebookId, 
						response.highscores[i].userId, 
						idToNames[''+facebookId], 
						response.highscores[i].score);
				
				// use idToNames to later filter out the ones that we already displayed
				idToNames[''+facebookId] = null;
				// do the current user have a score?
				if (response.highscores[i].userId == user.id)
					iHaveScore = true;
			}

			function compare(a,b) {
				if (a.score < b.score)
					return -1;
				if (a.score > b.score)
					return 1;
				return 0;
			}
			
			this.highscores.sort(compare);
			
			// Now add the ones that didn't have a score on this track, start with current user
			if (!iHaveScore) {
				this.highscores[i] = new Highscore(
						user.facebookId, 
						user.id, 
						idToNames[''+user.facebookId], 
						null);
				i++;
				idToNames[''+user.facebookId] = null; 
			}
			for (var a = 0 ; a < facebookIds.length ; a++) {
				facebookId = facebookIds[a];
				if ( idToNames[''+facebookId] !== null ) {
					this.highscores[i] = new Highscore(
							facebookId, 
							facebookId != user.facebookId ? null : user.id, 
							idToNames[''+facebookId], 
							null);
					i++;
				}
			}						

			this.updateWithHighscores(this.highscores, onUpdated);

		}.bind(this));

		req.send({
			boardId : this.board,
			facebookIds : facebookIds
		});

		return this;
	},
	/**
	 * Call this if you already have all the highscores (sorted) and just want to display them
	 */
	updateWithHighscores: function(highscores, onUpdated) {
		console.log('Running FriendLeaderboard.updateWithHighscores()');
		this.highscores = highscores;
		// Update the look
		this.table = new Element('table', {
			id : 'leaderboard',
			cellspacing : '0px',							
			cellpadding : '0px'							
		});

		this.table.grab(new Element('thead').
				grab(new Element('tr').
						grab(new Element('th', {
							colspan : "6",
							text : this.name
						}))
				)
		);
		this.rowsContainer = new Element('tbody');

		if (this.highscores.length > 0)
			this.highscores.each(this.makeOneRow,this);
		else
			this.rowsContainer.appendChild(new Element('a', {
				'class' : 'more',
				text : 'No highscores yet? Go for it, play some...'
			}));			

		this.table.appendChild(this.rowsContainer);

		clearChildren(this.element);

		this.element.appendChild(this.table);
		
		if (config.offline) {
			if (config.userMayRetryConnection) {
				this.makeRetryConnectionFooter();
			} else {
				this.makeLoginFooter();
			}
		} else {
			this.makeInviteFriendsFooter();
		}

		if (isFunction(onUpdated))   // More to do?
			onUpdated();
		else
			hideLoading();	// no, hide loading image

		return this;
	},
	makeInviteFriendsFooter: function() {
		var inviteLinkObj = {
			action : 3,
			params : {
				message : 'Come play this BMX racing game',
				title : 'Invite friends to play with you'
			}
		}; 
		this.element.appendChild( new Element('a', {
			href : 'game:'+ JSON.encode(inviteLinkObj),
			'class' : 'more',
			text : 'Tap here to invite more friends to play...'				
		}));		
	},
	makeLoginFooter: function() {
		var loginObj = {
				action : 7 // Login with facebook
		}; 
		
		var footerArea = new Element('div', {'class': 'more'});
		
		footerArea.appendChild(new Element('p', {
			text: 'Login to play against your friends\' record times!'
		}));
		
		footerArea.appendChild(new Element('a', {
			href : 'game:'+ JSON.encode(loginObj),
			'class' : 'fb_login_button'				
		}));		
		
		this.element.appendChild(footerArea);
	},
	makeRetryConnectionFooter: function() {
		var retryObj = {
				action : 5 // Retry connection
		}; 
		
		var footerArea = new Element('div', {'class': 'more offline_box'});

		footerArea.appendChild(new Element('h1', {
			'class': 'offline_header',
			text: 'Offline'
		}));		
		
		footerArea.appendChild(new Element('p', {
			text: 'Enable your Internet connection to play against your friends\' record times.'
		}));
		
		var retryBar = new Element('div', {
			id: 'retry_connection_bar'
		});
		footerArea.appendChild(retryBar);
		
		retryBar.appendChild(new Element('a', {
			href : 'game:'+ JSON.encode(retryObj),
			id: 'retry_connection_button'
		}));
		new NoClickDelay(retryBar);
		
		var retryLoader = new Element('div', {
			id: 'retry_connection_loader'
		});
		retryLoader.setStyle('display','none');
		retryBar.appendChild(retryLoader);
		
		this.element.appendChild(footerArea);
	},	
	makeOneRow: function(highscore, index) {
		var hasScore = (highscore.score !== null);
		var rowContainer = new Element('tr');

		var isMyself = (highscore.userId == '-1') || (highscore.userId == user.id) || (highscore.facebookId == user.facebookId);
		
        var position;
        var picture;
        var pictureFrame;
        var pictureImg;
        var name;
        var score;
		var attackLink;
        var attack;
        var watchLink;
        var watch;
        
		if ( hasScore ) {
			var notAvailable = (config.offline && !isMyself);
			
			position = new Element('td', {
				'class' : 'position',
				text : '#' + (index+1)
			});

			picture = new Element('td', {
				'class' : 'profilepicture'
			});
			pictureFrame = new Element('div', {
				'class' : 'frame'
			});			
			if (highscore.pictureURL) {
				pictureImg = new Element('div', {
					'class' : 'picture',
					styles: {
						background: 'url("' + highscore.pictureURL + '")'	
					}
				});
				pictureFrame.appendChild(pictureImg);
			}
			picture.appendChild(pictureFrame);

			name = new Element('td', {
				'class' : 'name',
				text : highscore.name
            });

			var time = new Date();
			time.clearTime();
			time.set('ms', highscore.score);

			score = new Element('td', {
				'class' : 'score',
				text : '' + time.format('racetime')
			});

			if (notAvailable) {
				attackLink = new Element('div', {
					'class' : 'button button_disabled'
				});
				attack = new Element('td', {
					'class' : 'play'
				}).grab(attackLink);
	
				watchLink = new Element('div', {
					'class' : 'button watch button_disabled'			
				}); 
				watch = new Element('td', {
					'class' : 'watch'
				}).grab(watchLink);				
			} else {	
				attackLink = new Element('a', {
					'class' : 'button',
					href : 'game:'+ JSON.encode({
						"action" : 1,
						"boardId": this.board, 
						"userId" : highscore.userId
					})			
				});
				attack = new Element('td', {
					'class' : 'play'
				}).grab(attackLink);
				new NoClickDelay(attack);
	
				watchLink = new Element('a', {
					'class' : 'button watch',
					href : 'game:'+ JSON.encode({
						"action" : 2,
						"boardId": this.board, 
						"userId" : highscore.userId
					})			
				}); 
				watch = new Element('td', {
					'class' : 'watch'
				}).grab(watchLink);
				new NoClickDelay(watch);
			}
			
			rowContainer.appendChild(position);
			rowContainer.appendChild(picture);
			rowContainer.appendChild(name);
			rowContainer.appendChild(score);
			rowContainer.appendChild(attack);
			rowContainer.appendChild(watch);
		} else {
			position = new Element('td', {'class' : 'position'});
			picture = new Element('td', {
				'class' : 'profilepicture'
			});
			pictureFrame = new Element('div', {
				'class' : 'frame'
			});			
			if (highscore.pictureURL) {
				pictureImg = new Element('div', {
					'class' : 'picture',
					styles: {
						background: 'url("' + highscore.pictureURL + '")'	
					}
				});
				pictureFrame.appendChild(pictureImg);
			}
			picture.appendChild(pictureFrame);
			name = new Element('td', {
				'class' : 'name',
				html : highscore.name
			});
			if (!isMyself) {
				// No score, and it's not me. Show 'remind' option
				score = new Element('td', {
					'class' : 'name',
					colspan : '3' 
				}).grab(new Element('small').grab(new Element('a', {
					href : 'game:'+ JSON.encode(
							{action : 3,
								params : { 
									message : 'You haven\'t played the track ' + this.name + ' yet. Can you beat my score?',
									title : 'Remind ' + highscore.name + ' to play',
									to : highscore.facebookId
								}
							}),
					html : 'Has not played this track yet,<br/><u>tap here</u> to remind'
				})));
				rowContainer.appendChild(position);
				rowContainer.appendChild(picture);
				rowContainer.appendChild(name);
				rowContainer.appendChild(score);
			} else {
				// No score, but it's me. Show 'play' button
				score = new Element('td', {
					'class' : 'score',
					text : ''
				});
				attackLink = new Element('a', {
					'class' : 'button',
					href : 'game:'+ JSON.encode({
						"action" : 6,
						"boardId": this.board 
					})			
				});
				attack = new Element('td', {
					'class' : 'play'
				}).grab(attackLink);
				new NoClickDelay(attack);
				
				watchLink = new Element('div', {
					'class' : 'button watch button_disabled'			
				}); 
				watch = new Element('td', {
					'class' : 'watch'
				}).grab(watchLink);				

				rowContainer.appendChild(position);
				rowContainer.appendChild(picture);
				rowContainer.appendChild(name);
				rowContainer.appendChild(score);
				rowContainer.appendChild(attack);
				rowContainer.appendChild(watch);
			}
		}
		this.rowsContainer.appendChild(rowContainer);

	}
});
