/**
 * 
 * How to use:
 * 
 * First perform setup, then set a function to call when setup is ready
 * 
 * 1) You can perform setup by calling setup(userId, userName, password, facebookId, accessToken, gameId)
 * or by passing a querystring with those parameters
 * 
 * 2) Use display(func) to set stuff to show once init is done
 * 
 * 
 * 
 * How to test in browser:
 * 
 * 1) Use querystring:
 * ?browserTest=true&
 * accessToken=AAADIoDpHEtkBAIPDL0wbtRCesyTh6nhTZA81qfESpP87KZBU1E8fChzujrnpr32Ntq89aTeB5ReZB3RupPuD9Lghti5T9t0fjQ65QqKnhKSptLYBZBB0
 * 
 * 2) Use display(func) to set stuff to show once init is done. Call from index.html or the command console
 * 
 * Get access_token from 
 * https://developers.facebook.com/tools/access_token/
 * 
 * 
 * How to handle exceptions:
 * 
 * define a function called handleException that takes two args, the type and the message. Stupid example:
 * 
 * function handleException(type, message) {
 *       alert(message);
 * }
 * 
 * type could be: WRONG_PARAMETERS  (see AbstractResponse.java in com.turborilla.net.protocol.response, and there's also FACEBOOK_ERROR and FACEBOOK_TIMEOUT)
 * message could be: Wrong facebook id passed from client 
 *			
 * The exception handler should probably send some or all exceptions on to Obj-C or other native code.
 * 
 */

var requestTimeout = 30000; // 30 second timeout

var body = document.id('body');
var user = new User();

var facebookId;
var urlParams = {};

var onInitedRunThis;
var pageLoaded = false;
var onPageLoaded;

// Show stuff when ready, call this from the game. Whenever, it waits until it's ready. 
// func is a function that returns an object that has an element (FriendLeaderboard for example)
// that way FriendLeaderboard won't be inited until everything is ready
//
// ex.: display(function() { return new FriendLeaderboard('tutorial-0','tutorial').update(); });
function display(func) {
	try {
		showLoading();
	
		if (!isFunction(func))
			throw new Error('The argument to display() must be a function');

		var container = $('wrap');
		if (!container)
			throw new Error('The document must contain an element with id: wrap');

		// Wait until everything is inited, then start showing stuff
		executeWhenReady(function() {
			var displayable = func();
			if (!displayable || !displayable.element)
				throw new Error('The argument to display() must be a function and that function must return an object that has an element');

			// Remove previous stuff
			clearChildren($('wrap'));
			
			// Add the new stuff
			$('wrap').appendChild(displayable.element);
		});
		
	} catch(e) {
		handle(e);
	} 
}

function clearChildren(element) {
	Array.each( element.getChildren(), function(item, index) {
		item.dispose();
	});
}

// Wait until the whole page is loaded
window.addEvent('load', function() {
	try {
		console.log('Page fully loaded');
		pageLoaded = true;
		
		if (isFunction(onPageLoaded))
			onPageLoaded();
		
		if (config.offline) {
			console.log('We are offline');
			onInitReady();
		} else {
			console.log('We are online. initing facebook...');
			initFacebook();
		}
	} catch(e) {
		handle(e);
	}
});

// Try to set everything up from the querystring params
function onParsedUrlParams() {
	
	// Console logging in ObjC
	// Do we need to stop this when using SSL? (does it give insecure warnings)
	if (!urlParams.browserTest) {
		console = {};
		console.log = function(log) {
			if (typeof(log)!='string') {
				try {
					var str = JSON.encode(log);
					log = str;
				} catch (e) { }
			}
			var iframe = document.createElement("IFRAME");
			iframe.setAttribute("src", "ios-log:#iOS#" + log);
			document.documentElement.appendChild(iframe);
			iframe.parentNode.removeChild(iframe);
			iframe = null;    
		};
		console.debug = console.log;
		console.info = console.log;
		console.warn = console.log;
		console.error = console.log;
	}
	
	// Setup
	if (urlParams.testOfflineLeaderboard) {
		config.offline = true;
	}
	else if (
			urlParams.userId &&
			urlParams.userName &&
			urlParams.password &&	
			urlParams.facebookId &&
			urlParams.accessToken
			) {
		
		setup(urlParams.userId,urlParams.userName,urlParams.password,urlParams.facebookId,urlParams.accessToken,urlParams.gameId,urlParams.platform);
		console.log('Client code set up from query string params. We are good to go');
		
	} else if (urlParams.browserTest) {
		setTimeout(function() {main(urlParams.accessToken, onInitReady);}, 500);
		
	} else {
		config.offline = true;
		console.log('Could not set client code up from query string params. setup(userId, userName, password, facebookId, accessToken, gameId) needs to be called');
	}
}

// This is called when GameRequest times out 
// globalOnTimeout('FB.api/me', true);
function globalOnTimeout(what, facebook) {
	console.log('Timed out when trying to access ' + what);
	config.offline = true;
	config.userMayRetryConnection = true;
	var name = facebook ? 'FACEBOOK_TIMEOUT' : 'TIMEOUT';
	setTimeout( function() {  // put it in a timeout, because of it is a facebook exception we might want to keep running scripts even after the timeout
		handle(new Error('Timed out when trying to access ' + what).setName(name));
	}, 100);			
}

// After loading the page and calling setup(), call this function with a function pointer to what to execute when everything is ready
function executeWhenReady(functionToExecute) {
	if (isInited()) {
		try {
			functionToExecute();
		} catch(e) {
			handle(e);
		}
	} else {
		onInitedRunThis = functionToExecute;
	}
}

//Call this from the phone web view if not using querystring
function setup(userId, userName, password, facebookId, accessToken, gameId, platform) {
	try {
		if (gameId !== undefined)
			config.game = gameId;
		
		if (platform !== undefined)
			config.platform = platform;

		user.init(userId, password, userName);
		user.facebookId = facebookId;
		user.accessToken = accessToken;
		token.access_token = accessToken;	
	} catch(e) {
		handle(e);
	}
	onInitReady();
}

function setAccessToken(accessToken) {
	if (user !== undefined)
		user.accessToken = accessToken;

	token.access_token = accessToken;	
}

var wasInited = false;

function onInitReady() {
	if (isFunction(onInitedRunThis)) 
		console.log('onInitReady() : There is a function waiting to be run when everything is inited...');
	else
		console.log('onInitReady() : No function waiting for isInited().');
		
	if (!isInited()) 
		console.log('onInitReady() : not inited yet.');
	else if (wasInited) 
		console.log('onInitReady() : already inited, not running \'onInitedRunThis\' again.');
	else {
		console.log('onInitReady() : everything is inited.');
		wasInited = true;

		// Call the onInited function
		if (isFunction(onInitedRunThis)) {
			var temp = onInitedRunThis;
			onInitedRunThis = null;
			try {
				temp();
			} catch(err) {
				handle(err);
			}
		}
		
		// If this isn't a browser test send a signal to the client announcing we're ready to receive
		if( !urlParams.browserTest ) 
			setTimeout(function() { 
				window.location.href='webview:{"action":"doneLoading"}';
				console.log('Webview sent \'doneLoading\' to Obj-C');
			}, 100);
	}
}

// are we logged on and ready?
function isInited() {
	if (config.offline)
		return true;
	else
		return user.loggedOn && token.access_token !== undefined && isFacebookInited;
}

//This main() is only for testing in browser
function main(accessToken, callWhenLoggedIn) {
	if (accessToken) {
		token.access_token = accessToken;


		facebook('/me',	function(response) {
			user.accessToken = accessToken;
			user.facebookId = response.id;
			user.login(callWhenLoggedIn);				
		});		
	}
}

// Facebook convenience function, will throw exception if something goes wrong
function facebook(fbfuncUrl, onSuccess) {
	console.log('Calling facebook: ' + fbfuncUrl);
	var timeoutHandle = new Timeout(globalOnTimeout, requestTimeout, this, ["FB.api/" + fbfuncUrl, true]);
		
	FB.api(fbfuncUrl, token, function(fbresponse) {
		try {
			if ( timeoutHandle.clear() ) {				
				// We haven't timed out, do stuff
				if (fbresponse.error) {
					// some error codes are known
					var code = fbresponse.error.code;
					if (code !== undefined && code !== null)
						if (code == '190')
							throw new Error(fbresponse.error.message).setName('INVALID_ACCESS_TOKEN').setCode(fbresponse.error.code);

					// unknown error code
					throw new Error(fbresponse.error.message).setName('FACEBOOK_ERROR').setCode(fbresponse.error.code);
				} else {
					console.log('Facebook response: ' + fbresponse);
					onSuccess(fbresponse);
				}
			}
		} catch (e) {
			handle(e);
		}
	});
}

// It is safe to call this before everything is loaded
function showLoading() {
	var showIt = function() { 
			var loading = $('loading');
			if (loading !== null)
				loading.removeProperty('hidden');
		};

	if (pageLoaded)
		showIt();
	else
		onPageLoaded = showIt;
}

function hideLoading() {
	var loading = $('loading');
	if (loading !== null)
		loading.setProperty('hidden','true');
}

//Extend exceptions
Error.implement('setName', function(newName) {
	this.name = newName;
	return this;
});
Error.implement('setCode', function(newCode) {
	this.code = newCode;
	return this;
});

//Handle exceptions
function handle(exception) {
	if (!urlParams.browserTest) {
		// Send to native
		window.location.href='game:{"action":4,"message":"' + exception.message + '","type":"' + exception.name + '","code":"' + exception.code + '"}';
	}
	else {
		if (isFunction(window.handleException) && exception) {
			// User-defined exception handler
			window.handleException(exception.name, exception.message, exception.code);
		} else {
			// No handler defined
			// just throw it and we'll get a nice stack trace in the console, 
			// unless it's supposed to be a message for the user from the server
			if (exception.name == 'USER_MESSAGE')
				alert(exception.message);
			else
				throw exception;
		}
	}
}

//check if a function is defined
function isFunction(functionToCheck) {
	 var getType = {};
	 return functionToCheck && getType.toString.call(functionToCheck) == '[object Function]';
}

//Init facebook, async. Run on 'load' event
var token = {};
var isFacebookInited = false;

function initFacebook() {
	// choose the correct domain to use for initing FB
	var channelHost = 'www.turborilla.net';
	var host = window.location.hostname;
	var appId;
	if (host) {
		if (host.indexOf('turborilla.net') != -1) {
			channelHost = 'www.turborilla.net';
			appId = '344289518973734';
		} else if (host.indexOf('localhost') != -1) {
			channelHost = 'localhost:8888';
		} else if (host.indexOf('staging.turborillanet.appspot.com') != -1) {
			channelHost = 'staging.turborillanet.appspot.com';
			appId = '220590498058969';
		} else if (host.indexOf('production.turborillanet.appspot.com') != -1) {
			channelHost = 'production.turborillanet.appspot.com';
			appId = '344289518973734';
		} else if (host.indexOf('dev1.turborillanet.appspot.com') != -1) {
			channelHost = 'dev1.turborillanet.appspot.com';
		}
	}
	console.log('Host = ' + window.location.hostname + ' -> and we are serving channel.html from: ' + channelHost);
	
	if (!isFacebookInited) {
		var timeoutHandle = new Timeout(globalOnTimeout, requestTimeout, this, ["FB.init", true]);

		window.fbAsyncInit = function() {
			FB.init({
				// appId      : appId, // App ID
				channelUrl : '//' + channelHost + '/channel.html' // Channel File
				//  , status     : true, // check login status
				//cookie     : true, // enable cookies to allow the server to access the session
				//xfbml      : false  // parse XFBML
			});
			if ( timeoutHandle.clear() ) {
				isFacebookInited = true;
				onInitReady();
			}
		};


		(function(d){
			var js, id = 'facebook-jssdk', ref = d.getElementsByTagName('script')[0];
			if (d.getElementById(id)) {return;}
			js = d.createElement('script'); js.id = id; js.async = true;
			js.src = "//connect.facebook.net/en_US/all.js";	
			ref.parentNode.insertBefore(js, ref);
		}(document));
	} else {
		onInitReady();
	}
}

// Prepare urlParams to find query string parameters
(function () {
    var e,
        a = /\+/g,
        r = /([^&=]+)=?([^&]*)/g,
        d = function (s) { return decodeURIComponent(s.replace(a, " ")); },
        q = window.location.search.substring(1);
        e = r.exec(q);
        
    while (e) {
       urlParams[d(e[1])] = d(e[2]);
       e = r.exec(q);
    }
    
    onParsedUrlParams();
})();

showLoading();

// Disable the tap delay on mobile safari
// Usage:  new NoClickDelay(document.getElementById('element'));
// element can be a container and it will work for all clickable elements in that container
// sets a 'pressed' class on the clicked element
function NoClickDelay(el) {
	this.element = typeof el == 'object' ? el : document.getElementById(el);
	if( window.Touch ) this.element.addEventListener('touchstart', this, false);
}

NoClickDelay.prototype = {
	handleEvent: function(e) {
		switch(e.type) {
			case 'touchstart': this.onTouchStart(e); break;
			case 'touchmove': this.onTouchMove(e); break;
			case 'touchend': this.onTouchEnd(e); break;
		}
	},

	onTouchStart: function(e) {
		e.preventDefault();
		this.moved = false;

		this.theTarget = document.elementFromPoint(e.targetTouches[0].clientX, e.targetTouches[0].clientY);
		if(this.theTarget.nodeType == 3) this.theTarget = theTarget.parentNode;
		this.theTarget.className+= ' pressed';

		this.element.addEventListener('touchmove', this, false);
		this.element.addEventListener('touchend', this, false);
	},

	onTouchMove: function(e) {
		this.moved = true;
		this.theTarget.className = this.theTarget.className.replace(/ ?pressed/gi, '');
	},

	onTouchEnd: function(e) {
		this.element.removeEventListener('touchmove', this, false);
		this.element.removeEventListener('touchend', this, false);

		if( !this.moved && this.theTarget ) {
			this.theTarget.className = this.theTarget.className.replace(/ ?pressed/gi, '');
			var theEvent = document.createEvent('MouseEvents');
			theEvent.initEvent('click', true, true);
			this.theTarget.dispatchEvent(theEvent);
		}

		this.theTarget = undefined;
	}
};